import React, { useCallback, useEffect, useId, useMemo, useState } from 'react';
import { useModal } from '@faceless-ui/modal';
import { useTranslation } from 'react-i18next';
import { getTranslation } from '../../../../utilities/getTranslation';
import { Drawer, DrawerToggler } from '../Drawer';
import { useRelatedCollections } from '../../forms/field-types/Relationship/AddNew/useRelatedCollections';
import { useEditDepth } from '../../utilities/EditDepth';
import { DocumentDrawerContent } from './DrawerContent';
import './index.scss';
export const baseClass = 'doc-drawer';
const formatDocumentDrawerSlug = ({ collectionSlug, id, depth, uuid, }) => `doc-drawer_${collectionSlug}_${depth}${id ? `_${id}` : ''}_${uuid}`;
export const DocumentDrawerToggler = ({ children, className, drawerSlug, id, collectionSlug, disabled, ...rest }) => {
    const { t, i18n } = useTranslation(['fields', 'general']);
    const [collectionConfig] = useRelatedCollections(collectionSlug);
    return (React.createElement(DrawerToggler, { slug: drawerSlug, formatSlug: false, className: [
            className,
            `${baseClass}__toggler`,
        ].filter(Boolean).join(' '), disabled: disabled, "aria-label": t(!id ? 'fields:addNewLabel' : 'general:editLabel', { label: getTranslation(collectionConfig.labels.singular, i18n) }), ...rest }, children));
};
export const DocumentDrawer = (props) => {
    const { drawerSlug } = props;
    return (React.createElement(Drawer, { slug: drawerSlug, formatSlug: false, className: baseClass },
        React.createElement(DocumentDrawerContent, { ...props })));
};
export const useDocumentDrawer = ({ id, collectionSlug }) => {
    const drawerDepth = useEditDepth();
    const uuid = useId();
    const { modalState, toggleModal, closeModal, openModal } = useModal();
    const [isOpen, setIsOpen] = useState(false);
    const drawerSlug = formatDocumentDrawerSlug({
        collectionSlug,
        id,
        depth: drawerDepth,
        uuid,
    });
    useEffect(() => {
        var _a;
        setIsOpen(Boolean((_a = modalState[drawerSlug]) === null || _a === void 0 ? void 0 : _a.isOpen));
    }, [modalState, drawerSlug]);
    const toggleDrawer = useCallback(() => {
        toggleModal(drawerSlug);
    }, [toggleModal, drawerSlug]);
    const closeDrawer = useCallback(() => {
        closeModal(drawerSlug);
    }, [closeModal, drawerSlug]);
    const openDrawer = useCallback(() => {
        openModal(drawerSlug);
    }, [openModal, drawerSlug]);
    const MemoizedDrawer = useMemo(() => {
        return ((props) => (React.createElement(DocumentDrawer, { ...props, collectionSlug: collectionSlug, id: id, drawerSlug: drawerSlug, key: drawerSlug })));
    }, [id, drawerSlug, collectionSlug]);
    const MemoizedDrawerToggler = useMemo(() => {
        return ((props) => (React.createElement(DocumentDrawerToggler, { ...props, id: id, collectionSlug: collectionSlug, drawerSlug: drawerSlug })));
    }, [id, drawerSlug, collectionSlug]);
    const MemoizedDrawerState = useMemo(() => ({
        drawerSlug,
        drawerDepth,
        isDrawerOpen: isOpen,
        toggleDrawer,
        closeDrawer,
        openDrawer,
    }), [drawerDepth, drawerSlug, isOpen, toggleDrawer, closeDrawer, openDrawer]);
    return [
        MemoizedDrawer,
        MemoizedDrawerToggler,
        MemoizedDrawerState,
    ];
};
