import React, { useCallback, useReducer, useState } from 'react';
import { ReactEditor, useFocused, useSelected, useSlateStatic } from 'slate-react';
import { useTranslation } from 'react-i18next';
import { Transforms } from 'slate';
import { useConfig } from '../../../../../../utilities/Config';
import usePayloadAPI from '../../../../../../../hooks/usePayloadAPI';
import { useDocumentDrawer } from '../../../../../../elements/DocumentDrawer';
import Button from '../../../../../../elements/Button';
import { useListDrawer } from '../../../../../../elements/ListDrawer';
import { getTranslation } from '../../../../../../../../utilities/getTranslation';
import './index.scss';
const baseClass = 'rich-text-relationship';
const initialParams = {
    depth: 0,
};
const Element = (props) => {
    var _a, _b, _c, _d, _e, _f;
    const { attributes, children, element, fieldProps, } = props;
    const { relationTo, value } = element;
    const { collections, serverURL, routes: { api } } = useConfig();
    const [enabledCollectionSlugs] = useState(() => collections.filter(({ admin: { enableRichTextRelationship } }) => enableRichTextRelationship).map(({ slug }) => slug));
    const [relatedCollection, setRelatedCollection] = useState(() => collections.find((coll) => coll.slug === relationTo));
    const selected = useSelected();
    const focused = useFocused();
    const { t, i18n } = useTranslation(['fields', 'general']);
    const editor = useSlateStatic();
    const [cacheBust, dispatchCacheBust] = useReducer((state) => state + 1, 0);
    const [{ data }, { setParams }] = usePayloadAPI(`${serverURL}${api}/${relatedCollection.slug}/${value === null || value === void 0 ? void 0 : value.id}`, { initialParams });
    const [DocumentDrawer, DocumentDrawerToggler, { closeDrawer, },] = useDocumentDrawer({
        collectionSlug: relatedCollection.slug,
        id: value === null || value === void 0 ? void 0 : value.id,
    });
    const [ListDrawer, ListDrawerToggler, { closeDrawer: closeListDrawer, },] = useListDrawer({
        collectionSlugs: enabledCollectionSlugs,
        selectedCollection: relatedCollection.slug,
    });
    const removeRelationship = useCallback(() => {
        const elementPath = ReactEditor.findPath(editor, element);
        Transforms.removeNodes(editor, { at: elementPath });
    }, [editor, element]);
    const updateRelationship = React.useCallback(({ doc }) => {
        const elementPath = ReactEditor.findPath(editor, element);
        Transforms.setNodes(editor, {
            type: 'relationship',
            value: { id: doc.id },
            relationTo: relatedCollection.slug,
            children: [
                { text: ' ' },
            ],
        }, { at: elementPath });
        setParams({
            ...initialParams,
            cacheBust, // do this to get the usePayloadAPI to re-fetch the data even though the URL string hasn't changed
        });
        closeDrawer();
        dispatchCacheBust();
    }, [editor, element, relatedCollection, cacheBust, setParams, closeDrawer]);
    const swapRelationship = React.useCallback(({ docID, collectionConfig }) => {
        const elementPath = ReactEditor.findPath(editor, element);
        Transforms.setNodes(editor, {
            type: 'relationship',
            value: { id: docID },
            relationTo: collectionConfig.slug,
            children: [
                { text: ' ' },
            ],
        }, { at: elementPath });
        setRelatedCollection(collections.find((coll) => coll.slug === collectionConfig.slug));
        setParams({
            ...initialParams,
            cacheBust, // do this to get the usePayloadAPI to re-fetch the data even though the URL string hasn't changed
        });
        closeListDrawer();
        dispatchCacheBust();
    }, [closeListDrawer, editor, element, cacheBust, setParams, collections]);
    return (React.createElement("div", { className: [
            baseClass,
            (selected && focused) && `${baseClass}--selected`,
        ].filter(Boolean).join(' '), contentEditable: false, ...attributes },
        React.createElement("div", { className: `${baseClass}__wrap` },
            React.createElement("p", { className: `${baseClass}__label` }, t('labelRelationship', { label: getTranslation(relatedCollection.labels.singular, i18n) })),
            React.createElement("p", { className: `${baseClass}__title` }, data[((_a = relatedCollection === null || relatedCollection === void 0 ? void 0 : relatedCollection.admin) === null || _a === void 0 ? void 0 : _a.useAsTitle) || 'id'])),
        React.createElement("div", { className: `${baseClass}__actions` },
            (value === null || value === void 0 ? void 0 : value.id) && (React.createElement(DocumentDrawerToggler, { className: `${baseClass}__toggler`, disabled: (_b = fieldProps === null || fieldProps === void 0 ? void 0 : fieldProps.admin) === null || _b === void 0 ? void 0 : _b.readOnly },
                React.createElement(Button, { icon: "edit", round: true, buttonStyle: "icon-label", el: "div", className: `${baseClass}__actionButton`, onClick: (e) => {
                        e.preventDefault();
                    }, tooltip: t('general:editLabel', { label: getTranslation(relatedCollection.labels.singular, i18n) }), disabled: (_c = fieldProps === null || fieldProps === void 0 ? void 0 : fieldProps.admin) === null || _c === void 0 ? void 0 : _c.readOnly }))),
            React.createElement(ListDrawerToggler, { disabled: (_d = fieldProps === null || fieldProps === void 0 ? void 0 : fieldProps.admin) === null || _d === void 0 ? void 0 : _d.readOnly },
                React.createElement(Button, { icon: "swap", round: true, buttonStyle: "icon-label", className: `${baseClass}__actionButton`, onClick: () => {
                        // do nothing
                    }, el: "div", tooltip: t('swapRelationship'), disabled: (_e = fieldProps === null || fieldProps === void 0 ? void 0 : fieldProps.admin) === null || _e === void 0 ? void 0 : _e.readOnly })),
            React.createElement(Button, { icon: "x", round: true, buttonStyle: "icon-label", className: `${baseClass}__actionButton`, onClick: (e) => {
                    e.preventDefault();
                    removeRelationship();
                }, tooltip: t('fields:removeRelationship'), disabled: (_f = fieldProps === null || fieldProps === void 0 ? void 0 : fieldProps.admin) === null || _f === void 0 ? void 0 : _f.readOnly })),
        (value === null || value === void 0 ? void 0 : value.id) && (React.createElement(DocumentDrawer, { onSave: updateRelationship })),
        React.createElement(ListDrawer, { onSelect: swapRelationship }),
        children));
};
export default Element;
