import React, { useCallback, useReducer, useState } from 'react';
import { Transforms } from 'slate';
import { ReactEditor, useSlateStatic, useFocused, useSelected } from 'slate-react';
import { useTranslation } from 'react-i18next';
import { useConfig } from '../../../../../../utilities/Config';
import usePayloadAPI from '../../../../../../../hooks/usePayloadAPI';
import FileGraphic from '../../../../../../graphics/File';
import useThumbnail from '../../../../../../../hooks/useThumbnail';
import Button from '../../../../../../elements/Button';
import { getTranslation } from '../../../../../../../../utilities/getTranslation';
import { useDocumentDrawer } from '../../../../../../elements/DocumentDrawer';
import { useListDrawer } from '../../../../../../elements/ListDrawer';
import './index.scss';
const baseClass = 'rich-text-upload';
const initialParams = {
    depth: 0,
};
const Element = ({ attributes, children, element }) => {
    var _a, _b, _c;
    const { relationTo, value, fieldProps, } = element;
    const { collections, serverURL, routes: { api } } = useConfig();
    const { t, i18n } = useTranslation('fields');
    const [cacheBust, dispatchCacheBust] = useReducer((state) => state + 1, 0);
    const [relatedCollection, setRelatedCollection] = useState(() => collections.find((coll) => coll.slug === relationTo));
    const [ListDrawer, ListDrawerToggler, { closeDrawer: closeListDrawer, },] = useListDrawer({
        uploads: true,
        selectedCollection: relatedCollection.slug,
    });
    const [DocumentDrawer, DocumentDrawerToggler, { closeDrawer, },] = useDocumentDrawer({
        collectionSlug: relatedCollection.slug,
        id: value === null || value === void 0 ? void 0 : value.id,
    });
    const editor = useSlateStatic();
    const selected = useSelected();
    const focused = useFocused();
    // Get the referenced document
    const [{ data }, { setParams }] = usePayloadAPI(`${serverURL}${api}/${relatedCollection.slug}/${value === null || value === void 0 ? void 0 : value.id}`, { initialParams });
    const thumbnailSRC = useThumbnail(relatedCollection, data);
    const removeUpload = useCallback(() => {
        const elementPath = ReactEditor.findPath(editor, element);
        Transforms.removeNodes(editor, { at: elementPath });
    }, [editor, element]);
    const updateUpload = useCallback((json) => {
        const { doc } = json;
        const newNode = {
            fields: doc,
        };
        const elementPath = ReactEditor.findPath(editor, element);
        Transforms.setNodes(editor, newNode, { at: elementPath });
        // setRelatedCollection(collections.find((coll) => coll.slug === collectionConfig.slug));
        setParams({
            ...initialParams,
            cacheBust, // do this to get the usePayloadAPI to re-fetch the data even though the URL string hasn't changed
        });
        dispatchCacheBust();
        closeDrawer();
    }, [editor, element, setParams, cacheBust, closeDrawer]);
    const swapUpload = React.useCallback(({ docID, collectionConfig }) => {
        const newNode = {
            type: 'upload',
            value: { id: docID },
            relationTo: collectionConfig.slug,
            children: [
                { text: ' ' },
            ],
        };
        const elementPath = ReactEditor.findPath(editor, element);
        setRelatedCollection(collections.find((coll) => coll.slug === collectionConfig.slug));
        Transforms.setNodes(editor, newNode, { at: elementPath });
        dispatchCacheBust();
        closeListDrawer();
    }, [closeListDrawer, editor, element, collections]);
    return (React.createElement("div", { className: [
            baseClass,
            (selected && focused) && `${baseClass}--selected`,
        ].filter(Boolean).join(' '), contentEditable: false, ...attributes },
        React.createElement("div", { className: `${baseClass}__card` },
            React.createElement("div", { className: `${baseClass}__topRow` },
                React.createElement("div", { className: `${baseClass}__thumbnail` }, thumbnailSRC ? (React.createElement("img", { src: thumbnailSRC, alt: data === null || data === void 0 ? void 0 : data.filename })) : (React.createElement(FileGraphic, null))),
                React.createElement("div", { className: `${baseClass}__topRowRightPanel` },
                    React.createElement("div", { className: `${baseClass}__collectionLabel` }, getTranslation(relatedCollection.labels.singular, i18n)),
                    React.createElement("div", { className: `${baseClass}__actions` },
                        (value === null || value === void 0 ? void 0 : value.id) && (React.createElement(DocumentDrawerToggler, { className: `${baseClass}__toggler` },
                            React.createElement(Button, { icon: "edit", round: true, buttonStyle: "icon-label", el: "div", className: `${baseClass}__actionButton`, onClick: (e) => {
                                    e.preventDefault();
                                }, tooltip: t('general:editLabel', { label: relatedCollection.labels.singular }), disabled: (_a = fieldProps === null || fieldProps === void 0 ? void 0 : fieldProps.admin) === null || _a === void 0 ? void 0 : _a.readOnly }))),
                        React.createElement(ListDrawerToggler, null,
                            React.createElement(Button, { icon: "swap", round: true, buttonStyle: "icon-label", className: `${baseClass}__actionButton`, onClick: () => {
                                    // do nothing
                                }, el: "div", tooltip: t('swapUpload'), disabled: (_b = fieldProps === null || fieldProps === void 0 ? void 0 : fieldProps.admin) === null || _b === void 0 ? void 0 : _b.readOnly })),
                        React.createElement(Button, { icon: "x", round: true, buttonStyle: "icon-label", className: `${baseClass}__actionButton`, onClick: (e) => {
                                e.preventDefault();
                                removeUpload();
                            }, tooltip: t('removeUpload'), disabled: (_c = fieldProps === null || fieldProps === void 0 ? void 0 : fieldProps.admin) === null || _c === void 0 ? void 0 : _c.readOnly })))),
            React.createElement("div", { className: `${baseClass}__bottomRow` },
                React.createElement("strong", null, data === null || data === void 0 ? void 0 : data.filename))),
        children,
        (value === null || value === void 0 ? void 0 : value.id) && (React.createElement(DocumentDrawer, { onSave: updateUpload })),
        React.createElement(ListDrawer, { onSelect: swapUpload })));
};
export default Element;
