import React, { useCallback, useEffect, useState } from 'react';
import { useTranslation } from 'react-i18next';
import Label from '../../Label';
import Error from '../../Error';
import FileDetails from '../../../elements/FileDetails';
import FieldDescription from '../../FieldDescription';
import { getTranslation } from '../../../../../utilities/getTranslation';
import { useDocumentDrawer } from '../../../elements/DocumentDrawer';
import { useListDrawer } from '../../../elements/ListDrawer';
import Button from '../../../elements/Button';
import './index.scss';
const baseClass = 'upload';
const UploadInput = (props) => {
    const { path, required, readOnly, style, className, width, description, label, relationTo, value, onChange, showError, serverURL = 'http://localhost:3000', api = '/api', collection, errorMessage, } = props;
    const { t, i18n } = useTranslation('fields');
    const [file, setFile] = useState(undefined);
    const [missingFile, setMissingFile] = useState(false);
    const [collectionSlugs] = useState([collection === null || collection === void 0 ? void 0 : collection.slug]);
    const [DocumentDrawer, DocumentDrawerToggler, { closeDrawer, },] = useDocumentDrawer({
        collectionSlug: collectionSlugs[0],
    });
    const [ListDrawer, ListDrawerToggler, { closeDrawer: closeListDrawer, },] = useListDrawer({
        collectionSlugs,
    });
    const classes = [
        'field-type',
        baseClass,
        className,
        showError && 'error',
        readOnly && 'read-only',
    ].filter(Boolean).join(' ');
    useEffect(() => {
        if (typeof value === 'string' && value !== '') {
            const fetchFile = async () => {
                const response = await fetch(`${serverURL}${api}/${relationTo}/${value}`, {
                    credentials: 'include',
                    headers: {
                        'Accept-Language': i18n.language,
                    },
                });
                if (response.ok) {
                    const json = await response.json();
                    setFile(json);
                }
                else {
                    setMissingFile(true);
                    setFile(undefined);
                }
            };
            fetchFile();
        }
        else {
            setFile(undefined);
        }
    }, [
        value,
        relationTo,
        api,
        serverURL,
        i18n,
    ]);
    const onSave = useCallback((args) => {
        setMissingFile(false);
        onChange(args.doc);
        closeDrawer();
    }, [onChange, closeDrawer]);
    const onSelect = useCallback((args) => {
        setMissingFile(false);
        onChange({
            id: args.docID,
        });
        closeListDrawer();
    }, [onChange, closeListDrawer]);
    return (React.createElement("div", { className: classes, style: {
            ...style,
            width,
        } },
        React.createElement(Error, { showError: showError, message: errorMessage }),
        React.createElement(Label, { htmlFor: `field-${path.replace(/\./gi, '__')}`, label: label, required: required }),
        (collection === null || collection === void 0 ? void 0 : collection.upload) && (React.createElement(React.Fragment, null,
            (file && !missingFile) && (React.createElement(FileDetails, { collection: collection, doc: file, handleRemove: () => {
                    onChange(null);
                } })),
            (!file || missingFile) && (React.createElement("div", { className: `${baseClass}__wrap` },
                React.createElement("div", { className: `${baseClass}__buttons` },
                    React.createElement(DocumentDrawerToggler, { className: `${baseClass}__toggler` },
                        React.createElement(Button, { buttonStyle: "secondary", el: "div" }, t('uploadNewLabel', { label: getTranslation(collection.labels.singular, i18n) }))),
                    React.createElement(ListDrawerToggler, { className: `${baseClass}__toggler` },
                        React.createElement(Button, { buttonStyle: "secondary", el: "div" }, t('chooseFromExisting')))))),
            React.createElement(FieldDescription, { value: file, description: description }))),
        React.createElement(DocumentDrawer, { onSave: onSave }),
        React.createElement(ListDrawer, { onSelect: onSelect })));
};
export default UploadInput;
